@@ -67,6 +67,30 @@ def async_options(function: Callable) -> Callable:
     return function
 
 
+def show_all_issues_option(function: Callable) -> Callable:
+    """Used for showing all issues (warnings, future-errors, and errors) instead of just errors"""
+    return click.option(
+        "--show-all", is_flag=True, default=False, help="If specified, prints warnings and future-errors"
+    )(function)
+
+
+def verbose_issues_option(function: Callable) -> Callable:
+    """Used for showing extra details issues might have that aren't shown by default"""
+    return click.option(
+        "--verbose-issues", is_flag=True, default=False, help="If specified, prints any extra details issues might have"
+    )(function)
+
+
+def dw_timeout_option(function: Callable) -> Callable:
+    """Used for timing out async dw validation results polling"""
+    return click.option(
+        "-t",
+        "--timeout",
+        type=int,
+        help="Sets the timeout in seconds to wait for individual data warehouse validations to finish. Defaults to None, meaning no timeout",
+    )(function)
+
+
 def config_validation_options(function: Callable) -> Callable:
     """Common options when validating or committing configs"""
     function = click.option(
@@ -76,6 +100,17 @@ def config_validation_options(function: Callable) -> Callable:
         default=os.getcwd(),
         help="Path to directory containing Transform yaml models",
     )(function)
+
+    function = click.option(
+        "--skip-dw",
+        is_flag=True,
+        default=False,
+        help="If specified, skips the data warehouse validations",
+    )(function)
+
+    function = show_all_issues_option(function)
+    function = verbose_issues_option(function)
+    function = dw_timeout_option(function)
     return function
 
 
@@ -88,6 +123,14 @@ def query_options(function: Callable) -> Callable:
         help="Path to directory containing Transform yaml models to execute query against",
     )(function)
 
+    function = click.option(
+        "--force-commit",
+        required=False,
+        is_flag=True,
+        default=False,
+        help="When using a local model (--config-dir), don't skip the commit process even if no changes are detected.",
+    )(function)
+
     function = click.option(
         "--cache-mode",
         type=click.Choice(
@@ -166,6 +209,14 @@ def query_options(function: Callable) -> Callable:
         callback=lambda ctx, param, value: PrintStyle(value) if value is not None else None,
         help="Choose your print style. Default: standard",
     )(function)
+
+    function = click.option(
+        "--trim",
+        required=False,
+        type=bool,
+        help="Trim incomplete time periods at the start and end of query results to avoid misleading data.",
+    )(function)
+
     return function
 
 
