@@ -1,9 +1,12 @@
 import json
+from pathlib import Path
 
 import click
+from click.shell_completion import ShellComplete, BashComplete, FishComplete, ZshComplete
+import shellingham
+
 import time
 import textwrap
-
 import pandas as pd
 from halo import Halo
 from rich.console import Console  # type: ignore
@@ -12,27 +15,32 @@ from datetime import datetime
 from packaging.version import parse
 
 from log_symbols import LogSymbols
-from termcolor import colored
 from update_checker import UpdateChecker
 
 from ..pretty_pandas import prettify
 
-from ..interfaces import DWValidationType
+from ..interfaces import AsyncDWValidationType
 
 from .cli_context import CLIContext
 from .cli_utils import (
+    dw_timeout_option,
     metrics_and_dimensions_options,
     query_options,
     config_validation_options,
     async_options,
     no_active_mql_servers,
+    show_all_issues_option,
     validate_query_args,
     enable_debug_log_file,
     get_query_logs_url_for_id,
     query_observability_options,
     exception_handler,
     start_end_time_options,
+    verbose_issues_option,
 )
+from metricflow.cli.main import _print_issues
+from metricflow.model.parsing.config_linter import ConfigLinter
+from metricflow.model.validations.validator_helpers import ModelValidationResults
 from ..models import (
     CacheMode,
     PercentChange,
@@ -43,7 +51,8 @@ from ..models import (
     TimeGranularity,
 )
 from .. import PACKAGE_NAME, __version__
-from ..constants import CLI_UPDATE_LAST_CHECKED, MAX_TIME_BEFORE_UPDATE
+from ..constants import CLI_UPDATE_LAST_CHECKED, MAX_TIME_BEFORE_UPDATE, TRANSFORM_PROD_APP
+from ..utils import get_cli_config_path
 
 pass_config = click.make_pass_decorator(CLIContext, ensure=True)
 
@@ -187,6 +196,9 @@ def setup(
     if mql_override:
         config.mql_server_url_config_override = mql_override
 
+    if config.just_authenticated:
+        config.cli_state.reset()
+
 
 @cli.command()
 @pass_config
@@ -228,6 +240,42 @@ def drop_cache(config: CLIContext, force: bool) -> None:
     spinner.succeed("💥 Successfully dropped MQL cache.")
 
 
+@cli.command()
+@pass_config
+@exception_handler
+@click.option(
+    "--metric-name",
+    required=False,
+    type=str,
+    help="Invalidate caches related to this metric",
+)
+@click.option("--skip-confirm", "-s", is_flag=True, default=False)
+def invalidate_caches(config: CLIContext, metric_name: Optional[str] = None, skip_confirm: bool = False) -> None:
+    """Invalidates caches
+
+    Invalidates caches related to the provided metric_name, if no metric is passed invalidate all caches.
+    """
+    cache_msg = f"caches related to {metric_name}" if metric_name else "all caches"
+    confirm_msg = (
+        click.style("WARNING", fg="yellow")
+        + f": Once invalidated, caches will be rebuilt from sources automatically, but until then, queries will be significantly slower.\nDo you want to continue to invalidate {cache_msg}?"
+    )
+    if not skip_confirm and not click.confirm(
+        confirm_msg,
+        abort=False,
+    ):
+        click.echo(f"❌ Abort invalidate {cache_msg} command.")
+        exit()
+
+    spinner = Halo(text="Initiating cache invalidation query...", spinner="dots")
+    spinner.start()
+
+    resp = config.mql.invalidate_cache_for_metric(metric_name) if metric_name else config.mql.invalidate_all_caches()
+    if not resp:
+        raise click.ClickException("‼️ Failed to invalidate caches.")
+    spinner.succeed(f"💥 Successfully invalidated {cache_msg}.")
+
+
 @cli.command()
 @pass_config
 @exception_handler
@@ -330,6 +378,7 @@ def query(
     limit: Optional[str] = None,
     cache_mode: Optional[CacheMode] = None,
     config_dir: Optional[str] = None,
+    force_commit: bool = False,
     start_time: Optional[str] = None,
     end_time: Optional[str] = None,
     as_table: Optional[str] = None,
@@ -340,13 +389,14 @@ def query(
     debug: bool = False,
     allow_dynamic_cache: bool = True,
     style: Optional[PrintStyle] = None,
+    trim: Optional[bool] = None,
 ) -> None:
     """Create a new MQL query, polls for completion and assembles a DataFrame from the response."""
     validate_query_args(limit)
 
     # Note: calling config.resolve_query_model_key may initiate a dialog if the user has a pinned model
     # Purposefully pulling this out before the spinner
-    model_key = config.resolve_query_model_key(config_dir)
+    model_key = config.resolve_query_model_key(config_dir, force_commit)
 
     start = time.time()
     spinner = Halo(text="Initiating query…", spinner="dots")
@@ -366,6 +416,7 @@ def query(
         allow_dynamic_cache=allow_dynamic_cache,
         start_time=start_time,
         end_time=end_time,
+        trim=trim,
     ).query_id
 
     spinner.succeed(f"Query initialized: {query_id}")
@@ -845,84 +896,164 @@ def list_queries(
             click.echo(f"• query_id={q.id}, status={q.status.value}")
 
 
-def _run_dw_validations(config: CLIContext, type: DWValidationType, model_id: Optional[int] = None) -> List[str]:
+def _run_dw_validations(
+    config: CLIContext,
+    dw_validation_type: AsyncDWValidationType,
+    model_id: Optional[int] = None,
+    timeout: Optional[int] = None,
+) -> List[str]:
     """Helper handles the calling of data warehouse issue generating functions"""
 
     spinner = Halo(
-        text=f"Validating {type.name.lower()} elements on model {model_id} against data warehouse...", spinner="dots"
+        text=f"Validating {dw_validation_type.name} elements on model {model_id} against data warehouse...",
+        spinner="dots",
     )
     spinner.start()
-    issues = config.mql.get_data_warehouse_issues(type=type, model_key_id=model_id)
-    if issues is not None and len(issues) == 0:
-        spinner.succeed(
-            f"🎉 Finished validating {type.name.lower()} elements on model {model_id} against data warehouse, no issues found"
+    job_id = config.mql.create_data_warehouse_validations_job(dw_validation_type=dw_validation_type)
+    results = config.mql.get_data_warehouse_validations_job_result(query_id=job_id, timeout=timeout)
+    if results.has_blocking_issues:
+        spinner.fail(
+            f"Breaking issues found when validating {dw_validation_type.name} elements against data warehouse ({results.summary()})"
         )
     else:
-        spinner.fail(
-            f"😬 Issues found when validating {type.name.lower()} elements on model {model_id} against data warehouse"
+        spinner.succeed(
+            f"🎉 Successfully validated {dw_validation_type.name} elements against data warehouse ({results.summary()})"
         )
-    return issues
+    return results
 
 
-def _data_warehouse_validations_runner(config: CLIContext, model_id: Optional[int] = None) -> bool:
+def _data_warehouse_validations_runner(
+    config: CLIContext,
+    model_id: Optional[int] = None,
+    show_all: bool = False,
+    additional_results: ModelValidationResults = ModelValidationResults(),
+    verbose_issues: bool = False,
+    timeout: Optional[int] = None,
+) -> bool:
     """Helper which calls the individual data warehouse validations to run and prints collected issues"""
     # the only reason we do this is so that the model id is available in terminal output
     if model_id is None and config.current_model is not None:
         model_id = config.current_model.id
 
-    issues: List[str] = []
-    issues += _run_dw_validations(config=config, type=DWValidationType.DATA_SOURCE, model_id=model_id)
-    issues += _run_dw_validations(config=config, type=DWValidationType.DIMENSION, model_id=model_id)
-    issues += _run_dw_validations(config=config, type=DWValidationType.METRIC, model_id=model_id)
+    list_results: List[ModelValidationResults] = [additional_results]
+    list_results.append(
+        _run_dw_validations(
+            config=config, dw_validation_type=AsyncDWValidationType.data_source, model_id=model_id, timeout=timeout
+        )
+    )
+    list_results.append(
+        _run_dw_validations(
+            config=config, dw_validation_type=AsyncDWValidationType.dimension, model_id=model_id, timeout=timeout
+        )
+    )
+    list_results.append(
+        _run_dw_validations(
+            config=config, dw_validation_type=AsyncDWValidationType.identifier, model_id=model_id, timeout=timeout
+        )
+    )
+    list_results.append(
+        _run_dw_validations(
+            config=config, dw_validation_type=AsyncDWValidationType.measure, model_id=model_id, timeout=timeout
+        )
+    )
+    list_results.append(
+        _run_dw_validations(
+            config=config, dw_validation_type=AsyncDWValidationType.metric, model_id=model_id, timeout=timeout
+        )
+    )
+    merged_results = ModelValidationResults.merge(list_results)
+    _print_issues(merged_results, show_non_blocking=show_all, verbose=verbose_issues)
 
-    print(colored("\n".join(issues), "red"))
-    return len(issues) == 0
+    return not merged_results.has_blocking_issues
 
 
 @cli.command()
 @click.option("--model-id", type=int, required=False, help="Specific model id to run")
+@show_all_issues_option
+@verbose_issues_option
+@dw_timeout_option
 @pass_config
 @exception_handler
-def data_warehouse_validations(config: CLIContext, model_id: Optional[int] = None) -> None:
+def data_warehouse_validations(
+    config: CLIContext,
+    model_id: Optional[int] = None,
+    show_all: bool = True,
+    verbose_issues: bool = False,
+    timeout: Optional[int] = None,
+) -> None:
     """Run data warehouse validations for a model, defaults to current model"""
-    _data_warehouse_validations_runner(config=config, model_id=model_id)
+    if not show_all:
+        print("(To see warnings and future-errors, run again with flag `--show-all`)")
+
+    _data_warehouse_validations_runner(
+        config=config, model_id=model_id, show_all=show_all, verbose_issues=verbose_issues, timeout=timeout
+    )
 
 
 @cli.command()
 @config_validation_options
-@click.option(
-    "--skip-dw",
-    is_flag=True,
-    default=False,
-    help="If specified, skips the data warehouse validations",
-)
 @pass_config
 @exception_handler
-def validate_configs(config: CLIContext, config_dir: str, skip_dw: bool = False) -> None:
+def validate_configs(
+    config: CLIContext,
+    config_dir: str,
+    skip_dw: bool = False,
+    show_all: bool = False,
+    verbose_issues: bool = False,
+    timeout: Optional[int] = None,
+) -> None:
     """Validate yaml configs found in specified config directory"""
-    spinner = Halo(text=f"Running parsing and semantic validations for configs in '{config_dir}'...", spinner="dots")
-    spinner.start()
+    if not show_all:
+        print("(To see warnings and future-errors, run again with flag `--show-all`)")
+
+    # Lint configs before uploading them to the backend for parsing and semantic validations
+    lint_spinner = Halo(text="Checking for YAML format issues", spinner="dots")
+    lint_spinner.start()
+
+    lint_results = ConfigLinter().lint_dir(config_dir)
+    if not lint_results.has_blocking_issues:
+        lint_spinner.succeed(f"🎉 Successfully linted config YAML files ({lint_results.summary()})")
+    else:
+        lint_spinner.fail(f"Breaking issues found in config YAML files ({lint_results.summary()})")
+        _print_issues(lint_results, show_non_blocking=show_all, verbose=verbose_issues)
+        return
+
+    # Upload, parse, and semantically validate the model via the backend
+    commit_spinner = Halo(
+        text=f"Running parsing and semantic validations for configs in '{config_dir}'...", spinner="dots"
+    )
+    commit_spinner.start()
 
     # Currently if this fails, an exception is thrown, and the calling function handles it with a decorator
-    model = config.mql.validate_configs(config_dir=config_dir)
-    spinner.succeed(
-        f"🦄 Successfully validated parsing and semantics of configs for commit ({model.commit}) in repo ({model.repository}), on branch ({model.branch})"
-    )
+    model, commit_results = config.mql.validate_configs(config_dir=config_dir)
+    combined_results = ModelValidationResults.merge([lint_results, commit_results])
+    if not commit_results.has_blocking_issues:
+        commit_spinner.succeed(
+            f"🦄 Successfully validated parsing and semantics of configs ({commit_results.summary()})"
+        )
+    else:
+        commit_spinner.fail(
+            f"Breaking issues found when validating parsing and semantics of configs ({commit_results.summary()})"
+        )
+        _print_issues(combined_results, show_non_blocking=show_all, verbose=verbose_issues)
+        return
 
     if not skip_dw:
-        _data_warehouse_validations_runner(config=config, model_id=model.id)
+        _data_warehouse_validations_runner(
+            config=config,
+            model_id=model.id,
+            show_all=show_all,
+            additional_results=combined_results,
+            verbose_issues=verbose_issues,
+            timeout=timeout,
+        )
 
 
 @cli.command()
 @config_validation_options
 @click.option("--pin", type=bool, required=False, help="Whether to pin the model or not")
 @click.option("--force-primary", is_flag=True, default=None, help="Make this model current")
-@click.option(
-    "--skip-dw",
-    is_flag=True,
-    default=False,
-    help="If specified, skips the data warehouse validations",
-)
+@click.option("--dev-ui", is_flag=True, default=False, help="See the model in the Transform app")
 @pass_config
 @click.pass_context
 def commit_configs(
@@ -931,20 +1062,57 @@ def commit_configs(
     config_dir: str,
     pin: Optional[bool] = None,
     force_primary: Optional[bool] = None,
+    dev_ui: bool = False,
     skip_dw: bool = False,
+    show_all: bool = False,
+    verbose_issues: bool = False,
+    timeout: Optional[int] = None,
 ) -> None:
     """Commit yaml configs found in specified config directory"""
-    spinner = Halo(text=f"Running parsing and semantic validations for configs in '{config_dir}'...", spinner="dots")
-    spinner.start()
+    if not show_all:
+        print("(To see warnings and future-errors, run again with flag `--show-all`)")
+
+    # Lint configs before uploading them to the backend for parsing and semantic validations
+    lint_spinner = Halo(text="Checking for YAML format issues", spinner="dots")
+    lint_spinner.start()
+
+    lint_results = ConfigLinter().lint_dir(config_dir)
+    if not lint_results.has_blocking_issues:
+        lint_spinner.succeed(f"🎉 Successfully linted config YAML files ({lint_results.summary()})")
+    else:
+        lint_spinner.fail(f"Breaking issues found in config YAML files ({lint_results.summary()})")
+        _print_issues(lint_results, show_non_blocking=show_all, verbose=verbose_issues)
+        return
+
+    # Upload, parse, and semantically validate the model via the backend
+    commit_spinner = Halo(
+        text=f"Running parsing and semantic validations for configs in '{config_dir}'...", spinner="dots"
+    )
+    commit_spinner.start()
 
     # Currently if this fails, an exception is thrown, and the calling function handles it with a decorator
-    model = config.mql.commit_configs(config_dir=config_dir)
-    spinner.succeed(
-        f"🦄 Successfully validated parsing and semantics of configs for commit ({model.commit}) in repo ({model.repository}), on branch ({model.branch})"
-    )
+    model, commit_results = config.mql.commit_configs(config_dir=config_dir)
+    combined_results = ModelValidationResults.merge([lint_results, commit_results])
+    if not commit_results.has_blocking_issues:
+        commit_spinner.succeed(
+            f"🦄 Successfully validated parsing and semantics of configs ({commit_results.summary()})"
+        )
+    else:
+        commit_spinner.fail(
+            f"Breaking issues found when validating parsing and semantics of configs ({commit_results.summary()})"
+        )
+        _print_issues(combined_results, show_non_blocking=show_all, verbose=verbose_issues)
+        return
 
     if not skip_dw:
-        success = _data_warehouse_validations_runner(config=config, model_id=model.id)
+        success = _data_warehouse_validations_runner(
+            config=config,
+            model_id=model.id,
+            show_all=show_all,
+            additional_results=combined_results,
+            verbose_issues=verbose_issues,
+            timeout=timeout,
+        )
         if not success:
             return
 
@@ -970,6 +1138,19 @@ def commit_configs(
                 """
             )
         )
+    dev_ui_link = f"{TRANSFORM_PROD_APP}/?model-id={model.id}"
+    if dev_ui:
+        click.launch(dev_ui_link)
+    else:
+        click.echo(
+            textwrap.dedent(
+                f"""\
+                📊 Test the model in the Transform App:
+
+                    {dev_ui_link}
+                """
+            )
+        )
 
 
 @cli.command()
@@ -1046,5 +1227,79 @@ def latest_mql_image(config: CLIContext) -> None:
     )
 
 
+_shell_properties = {
+    "bash": (".bashrc", BashComplete),
+    "zsh": (".zshrc", ZshComplete),
+    "fish": ("", FishComplete),
+}
+
+
+@cli.command()
+@pass_config
+@exception_handler
+def install_completion(config: CLIContext) -> None:
+    """Install command completion for the MQL CLI"""
+    try:
+        shell_name, _ = shellingham.detect_shell()
+    except shellingham.ShellDetectionFailure:
+        click.echo("Not running in a shell. Re-run `mql install completion` in a shell to install completion.")
+    if shell_name not in _shell_properties.keys():
+        click.echo(f"Completion for {shell_name} not yet supported.")
+        return
+
+    shell_config_file, complete_cls = _shell_properties[shell_name]
+    shell_complete: ShellComplete = complete_cls(None, {}, "mql", "_MQL_COMPLETE")
+    try:
+        completion_script = shell_complete.source()
+    except RuntimeError as e:
+        click.echo(e)
+        return
+
+    if shell_name == "fish":
+        fish_config_path = Path.home() / ".config" / "fish" / "completions"
+        fish_config_path.mkdir(parents=True, exist_ok=True)
+        script_path = fish_config_path / "mql.fish"
+        with script_path.open("w") as file:
+            file.write(completion_script)
+        click.echo("Added the completion script to ~/.config/fish/completions/mql.fish")
+        click.echo("To finish installation, restart your terminal.")
+        return
+
+    transform_config = get_cli_config_path()
+    transform_config.mkdir(parents=True, exist_ok=True)
+    script_path = transform_config / f"{shell_name}_completion.sh"
+    with script_path.open("w") as file:
+        file.write(completion_script)
+
+    shell_config_path = Path.home() / shell_config_file
+
+    if not shell_config_path.is_file():
+        click.echo(f"Error: {shell_config_file} not found in home directory.")
+        return
+
+    config_line = f"source {str(script_path.absolute())}"
+
+    already_installed = False
+    with shell_config_path.open() as file:
+        for line in file.readlines():
+            if config_line in line:
+                already_installed = True
+                break
+    if already_installed:
+        click.echo(f"Completion script already in {shell_config_file}.")
+        return
+    if not click.confirm(
+        f"To finish the install, we will add the completion script to {shell_config_file}. Would you like to continue?"
+    ):
+        click.echo(
+            f"You can finish the installation manually by adding `{config_line}` to {shell_config_file} and restarting your shell."
+        )
+        return
+    with shell_config_path.open("a") as file:
+        file.write(f"\n{config_line}\n")
+        click.echo(f"Added completion script to {shell_config_file}!")
+        click.echo("To finish installation and enable completion, restart your terminal.")
+
+
 if __name__ == "__main__":
     cli()
