@@ -13,3 +13,6 @@ CLI_UPDATE_LAST_CHECKED = "CLI_UPDATE_LAST_CHECKED"
 PREV_CONFIG_DIR_MODIFIED = "PREV_CONFIG_DIR_MODIFIED"
 PREV_CONFIG_DIR_PATH = "PREV_CONFIG_DIR_PATH"
 PREV_MODEL_ID = "PREV_MODEL_ID"
+
+TRANSFORM_PROD_API = "https://api.transformdata.io"
+TRANSFORM_PROD_APP = "https://app.transformdata.io"
