@@ -1,8 +1,8 @@
 import time
 
-from typing import Optional
+from typing import Optional, Union
 
-from .models import MqlQueryStatusResp
+from .models import MqlQueryStatusResp, QueryJobStatusResp
 
 
 class QueryRuntimeException(Exception):
@@ -32,7 +32,9 @@ class QueryRuntimeException(Exception):
         )
 
     @classmethod
-    def from_query_response(cls, resp: MqlQueryStatusResp) -> "QueryRuntimeException":  # noqa: D
+    def from_query_response(  # noqa: D
+        cls, resp: Union[MqlQueryStatusResp, QueryJobStatusResp]
+    ) -> "QueryRuntimeException":
         if not resp.is_complete or resp.is_successful:
             raise Exception("Unable to build exception from unfailed query")
         return cls(query_id=resp.query_id, msg=resp.error)
